(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2006 AIST.

   This file is written by Yutaka Oiwa in 2003-2006. *)

(* install various hooks into parser routines *)
open C_abstree
open Ctt_abstree
open C_pp

let dprintf l f = Debug.dprintf ~category:140 l f
let get_debug_flags () = Debug.get_debug_flags ~category:140

let trust_input_file_anyway = ref false
let check_location ~loc s = 
  let (_,_,_,loc_is_trustful) = loc in
  if loc_is_trustful || !trust_input_file_anyway then ()
  else
    failwith ("extension " ^ s ^ " is only allowed in system header.")

let translate_struct_by_extension ~loc ~ext def = 
  let o = ref [] in
  let () = match ext with
    [] -> ()
  | ["__fsc_attribute__", Eplist[Eplist l]] ->
      List.iter
	(function
	    Elist[Eident "named"; Estring str] ->
	      o := !o @ ["named", Estring str]
	  | Eident "external" ->
	      o := !o @ ["external", Elist []]
	  | _ -> 
	      failwith "unknown extension (42)") l
  | _ ->
      failwith "unknown extension"
  in
  { def with str_extension = !o }

let translate_external_extension ~loc ~ext ~sclass ~ty ~is_initialized = 
  let o = ref [] in
  let () = match ext with
    [] -> ()
  | ["__fsc_attribute__", Eplist[Eplist l]] ->
      List.iter
	(function
	  | Eident "external" -> begin
	      match ty.ct_ty with
		Tfunction _ ->
		  o := !o @ ["external", Elist []]
	      | _ -> failwith "__fsc_attribute__((external)) can only be applied to functions"
	  end
	  | Eident "emit_typeinfo" -> begin
	      match ty.ct_ty with
		Tstruct sid ->
		  Add_support_funcs.specially_emit_structs :=
		    sid :: !Add_support_funcs.specially_emit_structs;
		  o := !o @ ["emit_typeinfo", Elist []]
	      | _ ->
		  failwith "__fsc_attribute__((emit_typeinfo)) can only be applied to struct variables"
	  end
	  | _ -> 
	      failwith "unknown extension (34)") l
  | _ ->
      failwith "unknown extension"
  in
  !o

let remove_unused_filter ~ext _ id b = 
  dprintf 6 "declaration %s asked." id;
  if List.mem_assoc "emit_typeinfo" ext then 
    (dprintf 6 "declaration %s kept." id; true)
  else b

let install () = 
  C_lexer.extension_keywords := ["__fsc_attribute__"];
  C_lexer.use_failsafec_extension := true;
  C_typing.translate_struct_by_extension_hook := translate_struct_by_extension;
  C_typing.translate_external_extension_hook := translate_external_extension;
  Ctt_remove_unuseddecls.external_definition_filter_hook := remove_unused_filter;
  C_typing.fill_zero_initializer := false;
  C_typing.allow_calling_undefined_functions := true;
  ()


(* used by setjmp detection on Il2_reduce_local_variable *)

let fsc_default_il2_reduction_parameter = 
  { Il2_reduce_local_variable.is_setjmp = (fun i -> i = "__builtin_setjmp" || i = "__builtin_sigsetjmp"); 
    Il2_reduce_local_variable.forced_stack_alloc = false;
  }
